@@ -78,7 +78,6 @@ discharging
 """
 
 from __future__ import division  # python2 compatibility
-from time import time
 from re import findall
 from glob import iglob
 
@@ -119,9 +118,6 @@ class Py3status:
     threshold_degraded = 30
     threshold_full = 100
 
-    def __init__(self):
-        self.last_known_status = ''
-
     class Meta:
         deprecated = {
             'format_fix_unnamed_param': [
@@ -163,6 +159,7 @@ class Py3status:
         }
 
     def post_config_hook(self):
+        self.last_known_status = ''
         # Guess mode if not set
         if self.measurement_mode is None:
             if self.py3.check_commands(["acpi"]):
@@ -359,12 +356,17 @@ class Py3status:
 
                 time_remaining_seconds = self._hms_to_seconds(active_battery[
                     "time_remaining"])
-                rate_second_per_mah = time_remaining_seconds / (
-                    active_battery["capacity"] *
-                    (active_battery["percent_charged"] / 100))
-                time_remaining_seconds += inactive_battery["capacity"] * \
-                    inactive_battery["percent_charged"] / 100 * \
-                    rate_second_per_mah
+                try:
+                    rate_second_per_mah = time_remaining_seconds / (
+                        active_battery["capacity"] *
+                        (active_battery["percent_charged"] / 100))
+                    time_remaining_seconds += inactive_battery["capacity"] * \
+                        inactive_battery["percent_charged"] / 100 * \
+                        rate_second_per_mah
+                except ZeroDivisionError:
+                    # Either active or inactive battery has 0% charge
+                    time_remaining_seconds = 0
+                    rate_second_per_mah = 0
 
                 self.time_remaining = self._seconds_to_hms(
                     time_remaining_seconds)
@@ -391,8 +393,9 @@ class Py3status:
         if self.charging:
             self.icon = self.charging_character
         else:
-            self.icon = self.blocks[int(math.ceil(self.percent_charged / 100 *
-                                                  (len(self.blocks) - 1)))]
+            self.icon = self.blocks[min(len(self.blocks) - 1,
+                                        int(math.ceil(self.percent_charged / 100 *
+                                                      (len(self.blocks) - 1))))]
 
     def _update_full_text(self):
         self.full_text = self.py3.safe_format(
@@ -445,7 +448,7 @@ class Py3status:
         self.last_known_status = battery_status
 
     def _set_cache_timeout(self):
-        self.response['cached_until'] = time() + self.cache_timeout
+        self.response['cached_until'] = self.py3.time_in(self.cache_timeout)
 
 
 if __name__ == "__main__":
