@@ -21,9 +21,9 @@ Configuration parameters:
     icon_previous: text for the previous button in the button control panel (default '«')
     icon_stop: text for the stop button in the button control panel (default '◾')
     player_priority: priority of the players.
-            Keep in mind that the state has a higher priority than
-            player_priority. So when player_priority is "[mpd, bomi]" and mpd is
-            paused and bomi is playing than bomi wins. (default [])
+        Keep in mind that the state has a higher priority than
+        player_priority. So when player_priority is "[mpd, bomi]" and mpd is
+        paused and bomi is playing than bomi wins. (default [])
     state_pause: text for placeholder {state} when song is paused (default '▮')
     state_play: text for placeholder {state} when song is playing (default '▶')
     state_stop: text for placeholder {state} when song is stopped (default '◾')
@@ -112,6 +112,7 @@ SAMPLE OUTPUT
 from datetime import timedelta
 from time import time
 from gi.repository import GObject
+from gi.repository.GLib import GError
 from threading import Thread
 import re
 import sys
@@ -159,7 +160,7 @@ class Py3status:
     state_play = u'▶'
     state_stop = u'◾'
 
-    def __init__(self):
+    def post_config_hook(self):
         self._dbus = None
         self._data = {}
         self._control_states = {}
@@ -170,10 +171,36 @@ class Py3status:
         self._player = None
         self._player_details = {}
         self._tries = 0
-
-    def post_config_hook(self):
+        # start last
         self._dbus = SessionBus()
         self._start_listener()
+        self._states = {
+            'pause': {
+                'action': 'Pause',
+                'clickable': 'CanPause',
+                'icon': self.icon_pause
+            },
+            'play': {
+                'action': 'Play',
+                'clickable': 'CanPlay',
+                'icon': self.icon_play
+            },
+            'stop': {
+                'action': 'Stop',
+                'clickable': 'True',  # The MPRIS API lacks 'CanStop' function.
+                'icon': self.icon_stop
+            },
+            'next': {
+                'action': 'Next',
+                'clickable': 'CanGoNext',
+                'icon': self.icon_next
+            },
+            'previous': {
+                'action': 'Previous',
+                'clickable': 'CanGoPrevious',
+                'icon': self.icon_previous
+            }
+        }
 
     def _init_data(self):
         self._data = {
@@ -267,29 +294,9 @@ class Py3status:
         return (placeholders, color, update)
 
     def _get_control_states(self):
-        control_states = {
-            'pause': {'action': 'Pause',
-                      'clickable': 'CanPause',
-                      'icon': self.icon_pause},
-            'play': {'action': 'Play',
-                     'clickable': 'CanPlay',
-                     'icon': self.icon_play},
-            'stop': {'action': 'Stop',
-                     # Workaround: The MPRIS API has no CanStop function.
-                     'clickable': 'True',
-                     'icon': self.icon_stop},
-            'next': {'action': 'Next',
-                     'clickable': 'CanGoNext',
-                     'icon': self.icon_next},
-            'previous': {'action': 'Previous',
-                         'clickable': 'CanGoPrevious',
-                         'icon': self.icon_previous}
-        }
-
         state = 'pause' if self._data.get('state') == PLAYING else 'play'
-        control_states['toggle'] = control_states[state]
-
-        return control_states
+        self._states['toggle'] = self._states[state]
+        return self._states
 
     def _get_response_buttons(self):
         response = {}
@@ -377,6 +384,18 @@ class Py3status:
             status = data.get('PlaybackStatus')
             if status:
                 player = self._mpris_players[player_id]
+
+                # Note: Workaround. Since all players get noted if playback status
+                #       has been changed we have to check if we are the choosen one
+                try:
+                    dbus_status = player['_dbus_player'].PlaybackStatus
+                except GError:
+                    # Prevent errors when calling methods of deleted dbus objects
+                    return
+                if status != dbus_status:
+                    # FIXME: WE DON'T RECOGNIZE ANY TITLE CHANGE
+                    return
+
                 player['status'] = status
                 player['_state_priority'] = WORKING_STATES.index(status)
             self._set_player()
@@ -398,16 +417,19 @@ class Py3status:
                     p['name'] = self._mpris_names[p['identity']]
                     p['full_name'] = u'{} {}'.format(p['name'], p['index'])
 
-        status = player.PlaybackStatus
-        state_priority = WORKING_STATES.index(status)
         identity = player.Identity
+        name = self._mpris_names.get(identity)
+        if self.player_priority != [] and name not in self.player_priority \
+                and '*' not in self.player_priority:
+            return False
 
         if identity not in self._mpris_name_index:
             self._mpris_name_index[identity] = 0
 
+        status = player.PlaybackStatus
+        state_priority = WORKING_STATES.index(status)
         index = self._mpris_name_index[identity]
         self._mpris_name_index[identity] += 1
-        name = self._mpris_names.get(identity)
         subscription = player.PropertiesChanged.connect(self._player_monitor(player_id))
 
         self._mpris_players[player_id] = {
@@ -498,6 +520,8 @@ class Py3status:
         """
         Get the current output format and return it.
         """
+        if self._kill:
+            raise KeyboardInterrupt
         current_player_id = self._player_details.get('id')
         cached_until = self.py3.CACHE_FOREVER
 
@@ -514,9 +538,7 @@ class Py3status:
             (text, color, cached_until) = self._get_text()
             self._control_states = self._get_control_states()
             buttons = self._get_response_buttons()
-            composite = self.py3.build_composite(self.format,
-                                                 text,
-                                                 buttons)
+            composite = self.py3.safe_format(self.format, dict(text, **buttons))
 
         if (self._data.get('error_occurred') or
                 current_player_id != self._player_details.get('id')):
