@@ -3,12 +3,12 @@
 Display static text.
 
 Configuration parameters:
-    format: text that should be printed (default '')
+    format: display format for this module (default 'Hello, world!')
 
 @author frimdo ztracenastopa@centrum.cz
 
 SAMPLE OUTPUT
-{'full_text': 'Hello world!'}
+{'full_text': 'Hello, world!'}
 """
 
 
@@ -16,22 +16,18 @@ class Py3status:
     """
     """
     # available configuration parameters
-    format = ''
+    format = 'Hello, world!'
 
     def static_string(self):
-        response = {
+        return {
             'cached_until': self.py3.CACHE_FOREVER,
             'full_text': self.py3.safe_format(self.format),
         }
-        return response
 
 
 if __name__ == "__main__":
     """
     Run module in test mode.
     """
-    config = {
-        'format': 'Hello World!'
-    }
     from py3status.module_test import module_test
-    module_test(Py3status, config=config)
+    module_test(Py3status)
