@@ -8,14 +8,12 @@ Volume up/down and Toggle mute via mouse clicks can be easily added see
 example.
 
 Configuration parameters:
-    button_down: Button to click to decrease volume. Setting to 0 disables.
-        (default 0)
-    button_mute: Button to click to toggle mute. Setting to 0 disables.
-        (default 0)
-    button_up: Button to click to increase volume. Setting to 0 disables.
-        (default 0)
+    button_down: button to decrease volume (default None)
+    button_mute: button to toggle mute (default None)
+    button_up: button to increase volume (default None)
     cache_timeout: how often we refresh this module in seconds.
         (default 10)
+    card: Card to use. amixer supports this. (default None)
     channel: channel to track. Default value is backend dependent.
         (default None)
     command: Choose between "amixer", "pamixer" or "pactl".
@@ -24,11 +22,13 @@ Configuration parameters:
     device: Device to use. Defaults value is backend dependent
         (default None)
     format: Format of the output.
-        (default '♪: {percentage}%')
+        (default '[\?if=is_input 😮|♪]: {percentage}%')
     format_muted: Format of the output when the volume is muted.
-        (default '♪: muted')
+        (default '[\?if=is_input 😶|♪]: muted')
+    is_input: Is this an input device or an output device?
+        (default False)
     max_volume: Allow the volume to be increased past 100% if available.
-        pactl supports this (default 120)
+        pactl supports this. (default 120)
     thresholds: Threshold for percent volume.
         (default [(0, 'bad'), (20, 'degraded'), (50, 'good')])
     volume_delta: Percentage amount that the volume is increased or
@@ -73,10 +73,10 @@ Requires:
     pamixer: pulseaudio backend
 
 NOTE:
-        If you are changing volume state by external scripts etc and
-        want to refresh the module quicker than the i3status interval,
-        send a USR1 signal to py3status in the keybinding.
-        Example: killall -s USR1 py3status
+    If you are changing volume state by external scripts etc and
+    want to refresh the module quicker than the i3status interval,
+    send a USR1 signal to py3status in the keybinding.
+    Example: killall -s USR1 py3status
 
 @author <Jan T> <jans.tuomi@gmail.com>
 @license BSD
@@ -89,17 +89,22 @@ mute
 """
 
 import re
-from os import devnull
-from subprocess import check_output, call
+from os import devnull, environ as os_environ
+from subprocess import check_output, call, CalledProcessError
 
 
 class AudioBackend():
     def __init__(self, parent):
-        self.device = parent.device
+        self.card = parent.card
         self.channel = parent.channel
+        self.device = parent.device
+        self.is_input = parent.is_input
         self.parent = parent
         self.setup(parent)
 
+    def setup(self, parent):
+        raise NotImplementedError
+
     def run_cmd(self, cmd):
         with open(devnull, 'wb') as dn:
             return call(cmd, stdout=dn, stderr=dn)
@@ -107,14 +112,19 @@ class AudioBackend():
 
 class AmixerBackend(AudioBackend):
     def setup(self, parent):
+        if self.card is None:
+            self.card = '0'
+        if self.channel is None:
+            self.channel = 'Capture' if self.is_input else 'Master'
         if self.device is None:
             self.device = 'default'
-        if self.channel is None:
-            self.channel = 'Master'
-        self.cmd = ['amixer', '-q', '-D', self.device, 'sset', self.channel]
+        self.cmd = ['amixer', '-q', '-D', self.device,
+                    '-c', self.card, 'sset', self.channel]
+        self.get_volume_cmd = ['amixer', '-D', self.device,
+                               '-c', self.card, 'sget', self.channel]
 
     def get_volume(self):
-        output = check_output(['amixer', '-D', self.device, 'sget', self.channel]).decode('utf-8')
+        output = check_output(self.get_volume_cmd).decode('utf-8')
 
         # find percentage and status
         p = re.compile(r'\[(\d{1,3})%\].*\[(\w{2,3})\]')
@@ -144,10 +154,16 @@ class PamixerBackend(AudioBackend):
             self.device = "0"
         # Ignore channel
         self.channel = None
-        self.cmd = ["pamixer", "--sink", self.device]
+        self.cmd = ["pamixer", "--source" if self.is_input else "--sink", self.device]
 
     def get_volume(self):
-        perc = check_output(self.cmd + ["--get-volume"]).decode('utf-8').strip()
+        try:
+            perc = check_output(self.cmd + ["--get-volume"])
+        except CalledProcessError as cpe:
+            # pamixer throws error on zero percent. see #1135
+            perc = cpe.output
+
+        perc = perc.decode().strip()
         muted = (self.run_cmd(self.cmd + ["--get-mute"]) == 0)
         return perc, muted
 
@@ -163,16 +179,48 @@ class PamixerBackend(AudioBackend):
 
 class PactlBackend(AudioBackend):
     def setup(self, parent):
+        # get available device number if not specified
+        self.device_type = 'source' if self.is_input else 'sink'
+        self.device_type_pl = self.device_type + 's'
+        self.device_type_cap = self.device_type[0].upper() + self.device_type[1:]
+
         if self.device is None:
-            self.device = "0"
+            self.device = self.get_default_device()
+
+        self.english_env = dict(os_environ)
+        self.english_env['LC_ALL'] = 'C'
+
         self.max_volume = parent.max_volume
-        self.re_volume = re.compile(
-            r'Sink \#{}.*?Mute: (\w{{2,3}}).*?Volume:.*?(\d{{1,3}})\%'.format(self.device),
-            re.M | re.DOTALL
-        )
+        self.re_volume = re.compile(r'{} \#{}.*?Mute: (\w{{2,3}}).*?Volume:.*?(\d{{1,3}})\%'
+                                    .format(self.device_type_cap, self.device), re.M | re.DOTALL)
+
+    def get_default_device(self):
+        device_id = None
+
+        # Find the default device for the the device type
+        default_dev_pattern = re.compile(r'^Default {}: (.*)$'.format(self.device_type_cap))
+        for info_line in check_output(['pactl', 'info']).decode('utf-8').splitlines():
+            default_dev_match = default_dev_pattern.match(info_line)
+            if default_dev_match is not None:
+                device_id = default_dev_match.groups()[0]
+                break
+
+        # with the long gross id, find the associated number
+        if device_id is not None:
+            for line in check_output(['pactl', 'list', 'short', self.device_type_pl]) \
+                    .decode('utf-8').splitlines():
+                parts = line.split()
+                if len(parts) < 2:
+                    continue
+                if parts[1] == device_id:
+                    return parts[0]
+
+        raise RuntimeError('Failed to find default {} device.  Looked for {}'.format(
+            'input' if self.is_input else 'output', device_id))
 
     def get_volume(self):
-        output = check_output(['pactl', 'list', 'sinks']).decode('utf-8').strip()
+        output = check_output(
+            ['pactl', 'list', self.device_type_pl], env=self.english_env).decode('utf-8').strip()
         muted, perc = self.re_volume.search(output).groups()
 
         # muted should be 'on' or 'off'
@@ -189,34 +237,41 @@ class PactlBackend(AudioBackend):
             change = '{}%'.format(self.max_volume)
         else:
             change = '+{}%'.format(delta)
-        self.run_cmd(['pactl', '--', 'set-sink-volume', self.device, change])
+        self.run_cmd(['pactl', '--',
+                      'set-{}-volume'.format(self.device_type),
+                      self.device, change])
 
     def volume_down(self, delta):
-        self.run_cmd(['pactl', '--', 'set-sink-volume', self.device, '-{}%'.format(delta)])
+        self.run_cmd(['pactl', '--',
+                      'set-{}-volume'.format(self.device_type),
+                      self.device, '-{}%'.format(delta)])
 
     def toggle_mute(self):
-        self.run_cmd(['pactl', 'set-sink-mute', self.device, 'toggle'])
+        self.run_cmd(['pactl',
+                      'set-{}-mute'.format(self.device_type),
+                      self.device, 'toggle'])
 
 
 class Py3status:
     """
     """
     # available configuration parameters
-    button_down = 0
-    button_mute = 0
-    button_up = 0
+    button_down = None
+    button_mute = None
+    button_up = None
     cache_timeout = 10
+    card = None
     channel = None
     command = None
     device = None
-    format = u'♪: {percentage}%'
-    format_muted = u'♪: muted'
+    format = u'[\?if=is_input 😮|♪]: {percentage}%'
+    format_muted = u'[\?if=is_input 😶|♪]: muted'
+    is_input = False
     max_volume = 120
     thresholds = [(0, 'bad'), (20, 'degraded'), (50, 'good')]
     volume_delta = 5
 
     class Meta:
-
         def deprecate_function(config):
             # support old thresholds
             return {
@@ -244,11 +299,16 @@ class Py3status:
         }
 
     def post_config_hook(self):
-        # Guess command if not set
         if self.command is None:
             self.command = self.py3.check_commands(
-                ['amixer', 'pamixer', 'pactl']
-            )
+                ['amixer', 'pamixer', 'pactl'])
+
+        # turn integers to strings
+        if self.card is not None:
+            self.card = '%s' % self.card
+        if self.device is not None:
+            self.device = '%s' % self.device
+        self.volume_delta = int(self.volume_delta)
 
         if self.command == 'amixer':
             self.backend = AmixerBackend(self)
@@ -296,17 +356,16 @@ class Py3status:
         """
         button = event['button']
         # volume up
-        if self.button_up and button == self.button_up:
+        if button == self.button_up:
             self.backend.volume_up(self.volume_delta)
         # volume down
-        elif self.button_down and button == self.button_down:
+        elif button == self.button_down:
             self.backend.volume_down(self.volume_delta)
         # toggle mute
-        elif self.button_mute and button == self.button_mute:
+        elif button == self.button_mute:
             self.backend.toggle_mute()
 
 
-# test if run directly
 if __name__ == "__main__":
     """
     Run module in test mode.
