@@ -15,16 +15,26 @@ Configuration parameters:
     icon_off: show when Xscreensaver is not running (default 'XSCR')
     icon_on: show when Xscreensaver is running (default 'XSCR')
 
- Format placeholders:
+Format placeholders:
     {icon} Xscreensaver icon
 
+Color options:
+    color_on: Enabled, defaults to color_good
+    color_off: Disabled, defaults to color_bad
+
 @author neutronst4r <c7420{at}posteo{dot}net>, lasers
+
+SAMPLE OUTPUT
+{'color': '#00FF00', 'full_text': 'XSCR'}
+
+off
+{'color': '#FF0000', 'full_text': 'XSCR'}
 """
 
 from os import setpgrp
 from subprocess import Popen, PIPE
 
-STRING_UNAVAILABLE = "isn't installed"
+STRING_UNAVAILABLE = "not installed"
 
 
 class Py3status:
