@@ -1,18 +1,22 @@
 from __future__ import division
 
+import collections
 import os
 import sys
 import shlex
 
+from copy import deepcopy
 from fnmatch import fnmatch
 from math import log10
 from pprint import pformat
-from subprocess import Popen, PIPE
+from subprocess import Popen, PIPE, STDOUT
 from time import time
 
 from py3status import exceptions
 from py3status.formatter import Formatter, Composite
 from py3status.request import HttpResponse
+from py3status.util import Gradiants
+
 
 PY3_CACHE_FOREVER = -1
 PY3_LOG_ERROR = 'error'
@@ -83,37 +87,39 @@ class Py3:
     """Show as Warning"""
 
     # Shared by all Py3 Instances
-    _formatter = Formatter()
+    _formatter = None
+    _gradients = Gradiants()
     _none_color = NoneColor()
 
     # Exceptions
     Py3Exception = exceptions.Py3Exception
+    CommandError = exceptions.CommandError
     RequestException = exceptions.RequestException
+    RequestInvalidJSON = exceptions.RequestInvalidJSON
     RequestTimeout = exceptions.RequestTimeout
     RequestURLError = exceptions.RequestURLError
 
-    def __init__(self, module=None, i3s_config=None, py3status=None):
+    def __init__(self, module=None):
         self._audio = None
         self._config_setting = {}
         self._format_placeholders = {}
         self._format_placeholders_cache = {}
-        self._i3s_config = i3s_config or {}
-        self._module = module
         self._is_python_2 = sys.version_info < (3, 0)
+        self._module = module
         self._report_exception_cache = set()
         self._thresholds = None
+        self._threshold_gradients = {}
 
-        if py3status:
-            self._py3status_module = py3status
-
-        # we are running through the whole stack.
-        # If testing then module is None.
         if module:
+            self._i3s_config = module._py3_wrapper.config['py3_config']['general']
+            self._module_full_name = module.module_full_name
             self._output_modules = module._py3_wrapper.output_modules
-            if not i3s_config:
-                i3s_config = self._module.config['py3_config']['general']
-                self._i3s_config = i3s_config
             self._py3status_module = module.module_class
+            self._py3_wrapper = module._py3_wrapper
+            # create formatter we only if need one but want to pass py3_wrapper so
+            # that we can do logging etc.
+            if not self._formatter:
+                self.__class__._formatter = Formatter(module._py3_wrapper)
 
     def __getattr__(self, name):
         """
@@ -130,12 +136,9 @@ class Py3:
         try:
             return self._config_setting[name]
         except KeyError:
-            if self._module:
-                fn = self._module._py3_wrapper.get_config_attribute
-                param = fn(self._module.module_full_name, name)
-            else:
-                # running in test mode so config is not available
-                param = self._i3s_config.get(name, None)
+            fn = self._py3_wrapper.get_config_attribute
+            param = fn(self._module_full_name, name)
+
             # colors are special we want to make sure that we treat a color
             # that was explicitly set to None as a True value.  Ones that are
             # not set should be treated as None
@@ -191,8 +194,7 @@ class Py3:
         'position': list of places in i3bar, usually only one item
         'type': module type py3status/i3status
         """
-        if self._module:
-            return self._output_modules.get(module_name)
+        return self._output_modules.get(module_name)
 
     def _report_exception(self, msg, frame_skip=2):
         """
@@ -211,19 +213,18 @@ class Py3:
             return
         self._report_exception_cache.add(msg_hash)
 
-        if self._module:
-            # If we just report the error the traceback will end in the try
-            # except block that we are calling from.
-            # We want to show the traceback originating from the module that
-            # called the Py3 method so get the correct error frame and pass this
-            # along.
-            error_frame = sys._getframe(0)
-            while frame_skip:
-                error_frame = error_frame.f_back
-                frame_skip -= 1
-            self._module._py3_wrapper.report_exception(
-                msg, notify_user=False, error_frame=error_frame
-            )
+        # If we just report the error the traceback will end in the try
+        # except block that we are calling from.
+        # We want to show the traceback originating from the module that
+        # called the Py3 method so get the correct error frame and pass this
+        # along.
+        error_frame = sys._getframe(0)
+        while frame_skip:
+            error_frame = error_frame.f_back
+            frame_skip -= 1
+        self._py3_wrapper.report_exception(
+            msg, notify_user=False, error_frame=error_frame
+        )
 
     def error(self, msg, timeout=None):
         """
@@ -236,6 +237,73 @@ class Py3:
         """
         raise ModuleErrorException(msg, timeout)
 
+    def flatten_dict(self, d, delimiter='-', intermediates=False, parent_key=None):
+        """
+        Flatten a dictionary.
+
+        Values that are dictionaries are flattened using delimiter in between
+        (eg. parent-child)
+
+        Values that are lists are flattened using delimiter
+        followed by the index (eg. parent-0)
+
+        example:
+
+        .. code-block:: python
+
+            {
+                'fish_facts': {
+                    'sharks': 'Most will drown if they stop moving',
+                    'skates': 'More than 200 species',
+                },
+                'fruits': ['apple', 'peach', 'watermelon'],
+                'number': 52
+            }
+
+            # becomes
+
+            {
+                'fish_facts-sharks': 'Most will drown if they stop moving',
+                'fish_facts-skates': 'More than 200 species',
+                'fruits-0': 'apple',
+                'fruits-1': 'peach',
+                'fruits-2': 'watermelon',
+                'number': 52
+            }
+
+            # if intermediates is True then we also get unflattened elements
+            # as well as the flattened ones.
+
+            {
+                'fish_facts': {
+                    'sharks': 'Most will drown if they stop moving',
+                    'skates': 'More than 200 species',
+                },
+                'fish_facts-sharks': 'Most will drown if they stop moving',
+                'fish_facts-skates': 'More than 200 species',
+                'fruits': ['apple', 'peach', 'watermelon'],
+                'fruits-0': 'apple',
+                'fruits-1': 'peach',
+                'fruits-2': 'watermelon',
+                'number': 52
+            }
+        """
+        items = []
+        if isinstance(d, list):
+            d = dict(enumerate(d))
+        for k, v in d.items():
+            if parent_key:
+                k = u'{}{}{}'.format(parent_key, delimiter, k)
+            if intermediates:
+                items.append((k, v))
+            if isinstance(v, list):
+                v = dict(enumerate(v))
+            if isinstance(v, collections.Mapping):
+                items.extend(self.flatten_dict(v, delimiter, intermediates, str(k)).items())
+            else:
+                items.append((str(k), v))
+        return dict(items)
+
     def format_units(self, value, unit='B', optimal=5, auto=True, si=False):
         """
         Takes a value and formats it for user output, we can choose the unit to
@@ -360,8 +428,6 @@ class Py3:
         Returns True if the event name and instance match that of the module
         checking.
         """
-        if not self._module:
-            return False
 
         return (
             event.get('name') == self._module.module_name and
@@ -377,22 +443,25 @@ class Py3:
             self.LOG_ERROR, self.LOG_INFO, self.LOG_WARNING
         ], 'level must be LOG_ERROR, LOG_INFO or LOG_WARNING'
 
-        if self._module:
-            # nicely format logs if we can using pretty print
+        # nicely format logs if we can using pretty print
+        if isinstance(message, (dict, list, set, tuple)):
             message = pformat(message)
-            # start on new line if multi-line output
+        # start on new line if multi-line output
+        try:
             if '\n' in message:
                 message = '\n' + message
-            message = 'Module `{}`: {}'.format(
-                self._module.module_full_name, message)
-            self._module._py3_wrapper.log(message, level)
+        except:
+            pass
+        message = 'Module `{}`: {}'.format(
+            self._module.module_full_name, message)
+        self._py3_wrapper.log(message, level)
 
     def update(self, module_name=None):
         """
         Update a module.  If module_name is supplied the module of that
         name is updated.  Otherwise the module calling is updated.
         """
-        if not module_name and self._module:
+        if not module_name:
             return self._module.force_update()
         else:
             module_info = self._get_module_info(module_name)
@@ -407,14 +476,16 @@ class Py3:
         module_info = self._get_module_info(module_name)
         if module_info:
             output = module_info['module'].get_latest()
-        return output
+        # we do a deep copy so that any user does not change the actual output
+        # of the module.
+        return deepcopy(output)
 
     def trigger_event(self, module_name, event):
         """
         Trigger an event on a named module.
         """
-        if module_name and self._module:
-            self._module._py3_wrapper.events_thread.process_event(
+        if module_name:
+            self._py3_wrapper.events_thread.process_event(
                 module_name, event)
 
     def prevent_refresh(self):
@@ -423,8 +494,7 @@ class Py3:
         request that the module is not refreshed after the event. By default
         the module is updated after the on_click event has been processed.
         """
-        if self._module:
-            self._module.prevent_refresh = True
+        self._module.prevent_refresh = True
 
     def notify_user(self, msg, level='info', rate_limit=5):
         """
@@ -433,13 +503,14 @@ class Py3:
         rate_limit is the time period in seconds during which this message
         should not be repeated.
         """
-        if self._module:
-            # force unicode for python2 str
-            if self._is_python_2 and isinstance(msg, str):
-                msg = msg.decode('utf-8')
-            module_name = self._module.module_full_name
-            self._module._py3_wrapper.notify_user(
-                msg, level=level, rate_limit=rate_limit, module_name=module_name)
+        if isinstance(msg, Composite):
+            msg = msg.text()
+        # force unicode for python2 str
+        if self._is_python_2 and isinstance(msg, str):
+            msg = msg.decode('utf-8')
+        module_name = self._module.module_full_name
+        self._py3_wrapper.notify_user(
+            msg, level=level, rate_limit=rate_limit, module_name=module_name)
 
     def register_function(self, function_name, function):
         """
@@ -473,9 +544,8 @@ class Py3:
 
                 This function should only be used by containers.
         """
-        if self._module:
-            my_info = self._get_module_info(self._module.module_full_name)
-            my_info[function_name] = function
+        my_info = self._get_module_info(self._module.module_full_name)
+        my_info[function_name] = function
 
     def time_in(self, seconds=None, sync_to=None, offset=0):
         """
@@ -528,11 +598,12 @@ class Py3:
 
         return requested + offset
 
-    def format_contains(self, format_string, name):
+    def format_contains(self, format_string, names):
         """
-        Determines if ``format_string`` contains placeholder ``name``
+        Determines if ``format_string`` contains a placeholder string ``names``
+        or a list of placeholders ``names``.
 
-        ``name`` is tested against placeholders using fnmatch so the following
+        ``names`` is tested against placeholders using fnmatch so the following
         patterns can be used:
 
         .. code-block:: none
@@ -547,10 +618,14 @@ class Py3:
         will fail if the format string contains placeholder formatting
         eg ``'{placeholder:.2f}'``
         """
-
         # We cache things to prevent parsing the format_string more than needed
+        if isinstance(names, list):
+            key = str(names)
+        else:
+            key = names
+            names = [names]
         try:
-            return self._format_placeholders_cache[format_string][name]
+            return self._format_placeholders_cache[format_string][key]
         except KeyError:
             pass
 
@@ -560,15 +635,16 @@ class Py3:
         else:
             placeholders = self._format_placeholders[format_string]
 
-        result = False
-        for placeholder in placeholders:
-            if fnmatch(placeholder, name):
-                result = True
-                break
         if format_string not in self._format_placeholders_cache:
             self._format_placeholders_cache[format_string] = {}
-        self._format_placeholders_cache[format_string][name] = result
-        return result
+
+        for name in names:
+            for placeholder in placeholders:
+                if fnmatch(placeholder, name):
+                    self._format_placeholders_cache[format_string][key] = True
+                    return True
+        self._format_placeholders_cache[format_string][key] = False
+        return False
 
     def get_placeholders_list(self, format_string, match=None):
         """
@@ -603,6 +679,15 @@ class Py3:
                 found.append(placeholder)
         return found
 
+    def get_placeholder_formats_list(self, format_string):
+        """
+        Parses the format_string and returns a list of tuples
+        [(placeholder, format), ...].
+
+        eg ``'{placeholder:.2f}'`` will give ``[('placeholder', ':.2f')]``
+        """
+        return self._formatter.get_placeholder_formats_list(format_string)
+
     def safe_format(self, format_string, param_dict=None,
                     force_composite=False, attr_getter=None):
         """
@@ -767,27 +852,45 @@ class Py3:
         if isinstance(command, basestring):
             command = shlex.split(command)
         try:
-            return Popen(command, stdout=PIPE, stderr=PIPE).wait()
+            return Popen(
+                command, stdout=PIPE, stderr=PIPE, close_fds=True
+            ).wait()
         except Exception as e:
-            msg = "Command '{cmd}' {error}"
-            raise Exception(msg.format(cmd=command[0], error=e))
+            # make a pretty command for error loggings and...
+            if isinstance(command, basestring):
+                pretty_cmd = command
+            else:
+                pretty_cmd = ' '.join(command)
+            msg = 'Command `{cmd}` {error}'.format(cmd=pretty_cmd, error=e.errno)
+            raise exceptions.CommandError(msg, error_code=e.errno)
 
-    def command_output(self, command, shell=False):
+    def command_output(self, command, shell=False, capture_stderr=False):
         """
         Run a command and return its output as unicode.
         The command can either be supplied as a sequence or string.
 
-        An Exception is raised if an error occurs
+        :param command: command to run can be a str or list
+        :param shell: if `True` then command is run through the shell
+        :param capture_stderr: if `True` then STDERR is piped to STDOUT
+
+        A CommandError is raised if an error occurs
         """
-        # convert the command to sequence if a string
-        if isinstance(command, basestring):
+        # make a pretty command for error loggings and...
+        # convert the non-shell command to sequence if it is a string
+        if not shell and isinstance(command, basestring):
+            pretty_cmd = command
             command = shlex.split(command)
+        else:
+            pretty_cmd = ' '.join(command)
+
+        stderr = STDOUT if capture_stderr else PIPE
+
         try:
-            process = Popen(command, stdout=PIPE, stderr=PIPE,
+            process = Popen(command, stdout=PIPE, stderr=stderr, close_fds=True,
                             universal_newlines=True, shell=shell)
         except Exception as e:
-            msg = "Command '{cmd}' {error}"
-            raise Exception(msg.format(cmd=command[0], error=e))
+            msg = 'Command `{cmd}` {error}'.format(cmd=pretty_cmd, error=e)
+            raise exceptions.CommandError(msg, error_code=e.errno)
 
         output, error = process.communicate()
         if self._is_python_2:
@@ -801,13 +904,23 @@ class Py3:
             # reason is not entirely clear.
             if retcode == -15:
                 msg = 'Command `{cmd}` returned SIGTERM (ignoring)'
-                self.log(msg.format(cmd=command))
+                self.log(msg.format(cmd=pretty_cmd))
             else:
-                msg = "Command '{cmd}' returned non-zero exit status {error}"
-                raise Exception(msg.format(cmd=command[0], error=retcode))
+                msg = 'Command `{cmd}` returned non-zero exit status {error}'
+                output_oneline = output.replace('\n', ' ')
+                if output_oneline:
+                    msg += ' ({output})'
+                msg = msg.format(cmd=pretty_cmd, error=retcode, output=output_oneline)
+                raise exceptions.CommandError(
+                    msg, error_code=retcode, error=error, output=output
+                )
         if error:
-            msg = "Command '{cmd}' had error {error}"
-            raise Exception(msg.format(cmd=command[0], error=error))
+            msg = "Command '{cmd}' had error {error}".format(
+                cmd=pretty_cmd, error=error
+            )
+            raise exceptions.CommandError(
+                msg, error_code=retcode, error=error, output=output
+            )
         return output
 
     def play_sound(self, sound_file):
@@ -838,6 +951,9 @@ class Py3:
         threshold is needed for a module then the name can also be supplied.
         If the user has not supplied a named threshold but has defined a
         general one that will be used.
+
+        If the gradients config parameter is True then rather than sharp
+        thresholds we will use a gradient between the color values.
         """
         # If first run then process the threshold data.
         if self._thresholds is None:
@@ -853,13 +969,33 @@ class Py3:
         name_used = name
         if name_used not in self._thresholds:
             name_used = None
+        thresholds = self._thresholds.get(name_used)
+        if color is None and thresholds:
+            # if gradients are enabled then we use them
+            if self._get_config_setting('gradients'):
+                try:
+                    colors, minimum, maximum = self._threshold_gradients[name_used]
+                except KeyError:
+                    colors = self._gradients.make_threshold_gradient(self, thresholds)
+                    minimum = min(thresholds)[0]
+                    maximum = max(thresholds)[0]
+                    self._threshold_gradients[name_used] = (colors, minimum, maximum)
 
-        if color is None:
-            for threshold in self._thresholds.get(name_used, []):
-                if value >= threshold[0]:
-                    color = threshold[1]
-                else:
-                    break
+                if value < minimum:
+                    return colors[0]
+                if value > maximum:
+                    return colors[-1]
+                value -= minimum
+                col_index = int(((len(colors) - 1) / (maximum - minimum)) * value)
+                color = colors[col_index]
+
+            elif color is None:
+                color = thresholds[0][1]
+                for threshold in thresholds:
+                    if value >= threshold[0]:
+                        color = threshold[1]
+                    else:
+                        break
 
         # save color so it can be accessed via safe_format()
         if name:
@@ -885,7 +1021,7 @@ class Py3:
         :returns: HttpResponse
         """
 
-        # The aim of this function is to be a lmited lightweight replacement
+        # The aim of this function is to be a limited lightweight replacement
         # for the requests library but using only pythons standard libs.
 
         # IMPORTANT NOTICE
